"""Tiered Debugging Module"""

from ._base import set_level, set_stacklevel, lv1, lv2, lv3, lv4, lv5

__all__ = ["set_level", "set_stacklevel", "lv1", "lv2", "lv3", "lv4", "lv5"]
__version__ = "1.0.0"
